@@ -1,4 +1,4 @@
-/* $Header: /cvsroot/watchdog/watchdog/src/watchdog.c,v 1.4 2007/08/17 09:24:54 meskes Exp $ */
+/* $Header: /cvsroot/watchdog/watchdog/src/watchdog.c,v 1.5 2009/02/25 09:38:18 meskes Exp $ */
 
 /*************************************************************/
 /* Original version was an example in the kernel source tree */
@@ -23,6 +23,9 @@
 #include <arpa/inet.h>
 #include <sys/mman.h>
 #include <sys/wait.h>
+#include <sys/stat.h>
+#include <sys/types.h>
+#include <linux/watchdog.h>
 #define __USE_GNU
 #include <string.h>
 
@@ -44,9 +47,12 @@ static int no_act = FALSE;
 int verbose = FALSE;
 #endif				/* USE_SYSLOG */
 
+volatile sig_atomic_t _running = 1;
+
 #define ADMIN		"admin"
 #define CHANGE		"change"
 #define DEVICE		"watchdog-device"
+#define DEVICE_TIMEOUT	"watchdog-timeout"
 #define	FILENAME	"file"
 #define INTERFACE	"interface"
 #define INTERVAL	"interval"
@@ -74,6 +80,7 @@ int tint = 10, logtick = 1, ticker = 1, schedprio = 1;
 int maxload1 = 0, maxload5 = 0, maxload15 = 0, minpages = 0;
 int maxtemp = 120, hbstamps = 300, lastts, nrts;
 int pingcount = 3;
+int devtimeout = TIMER_MARGIN;
 char *tempname = NULL, *devname = NULL, *admin = "root", *progname;
 char *timestamps, *heartbeat;
 time_t timeout = 0;
@@ -87,9 +94,9 @@ static void usage(void)
 {
     fprintf(stderr, "%s version %d.%d, usage:\n", progname, MAJOR_VERSION, MINOR_VERSION);
 #if USE_SYSLOG
-    fprintf(stderr, "%s [-i <interval> [-f]] [-l <max load avg>] [-v] [-s] [-b] [-m <max temperature>]\n", progname);
+    fprintf(stderr, "%s [-f] [-c <config_file>] [-v] [-s] [-b] [-q]\n", progname);
 #else				/* USE_SYSLOG */
-    fprintf(stderr, "%s [-i <interval> [-f]] [-l <max load avg>] [-v] [-b] [-m <max temperature>]\n", progname);
+    fprintf(stderr, "%s [-f] [-c <config_file>] [-s] [-b] [-q]\n", progname);
 #endif				/* USE_SYSLOG */
     exit(1);
 }
@@ -105,7 +112,7 @@ static int sync_system(int sync_it)
 }
 
 /* execute repair binary */
-static int repair(char *rbinary, int result)
+static int repair(char *rbinary, int result, char *name)
 {
     pid_t child_pid;
     char parm[5];
@@ -128,7 +135,10 @@ static int repair(char *rbinary, int result)
         	exit (errno);
 
         /* else start binary */
-	execl(rbinary, rbinary, parm, NULL);
+	if (name == NULL)
+		execl(rbinary, rbinary, parm, NULL);
+	else
+		execl(rbinary, rbinary, parm, name, NULL);
 
 	/* execl should only return in case of an error */
 	/* so we return the reboot code */
@@ -166,7 +176,7 @@ static int repair(char *rbinary, int result)
 #endif				/* USE_SYSLOG */
 
 	if (ret == ERESET) /* repair script says force hard reset, we give it a try */
-		sleep(TIMER_MARGIN * 4);
+		sleep(devtimeout * 4);
 	
 	/* for all other errors or if we still live, we let shutdown handle it */
 	return (ret);
@@ -175,7 +185,7 @@ static int repair(char *rbinary, int result)
     return (ENOERR);
 }
 
-static void wd_action(int result, char *rbinary)
+static void wd_action(int result, char *rbinary, char *name)
 {
     /* if no-action flag set, do nothing */
     /* no error, keep on working */
@@ -184,7 +194,7 @@ static void wd_action(int result, char *rbinary)
 
     /* error that might be repairable */
     if (result != EREBOOT)
-	result = repair(rbinary, result);
+	result = repair(rbinary, result, name);
 
     /* if still error, reboot */
     if (result != ENOERR)
@@ -192,10 +202,10 @@ static void wd_action(int result, char *rbinary)
 
 }
 
-static void do_check(int res, char *rbinary)
+static void do_check(int res, char *rbinary, char *name)
 {
-    wd_action(res, rbinary);
-    wd_action(keep_alive(), rbinary);
+    wd_action(res, rbinary, name);
+    wd_action(keep_alive(), rbinary, NULL);
 }
 
 struct list *file = NULL, *target = NULL, *pidfile = NULL, *iface = NULL;
@@ -365,6 +375,11 @@ static void read_config(char *filename, char *progname)
 			devname = NULL;
 		else
 			devname = strdup(line + i);
+	    } else if (strncmp(line + i, DEVICE_TIMEOUT, strlen(DEVICE_TIMEOUT)) == 0) {
+		if (spool(line, &i, strlen(DEVICE_TIMEOUT)))
+			fprintf(stderr, "Ignoring invalid line in config file: %s ", line);
+		else
+			devtimeout = atol(line + i);
 	    } else if (strncmp(line + i, TEMP, strlen(TEMP)) == 0) {
 		if (spool(line, &i, strlen(TEMP)))
 			tempname = NULL;
@@ -508,7 +523,7 @@ int main(int argc, char *const argv[])
     if (tint < 0)
 	usage();
 
-    if (tint >= TIMER_MARGIN && !force) {
+    if (tint >= devtimeout && !force) {
 	fprintf(stderr, "%s error:\n", progname);
 	fprintf(stderr, "This interval length might reboot the system while the process sleeps!\n");
 	fprintf(stderr, "To force this interval length use the -f option.\n");
@@ -616,7 +631,7 @@ int main(int argc, char *const argv[])
     /* Log the starting message */
     openlog(progname, LOG_PID, LOG_DAEMON);
     syslog(LOG_INFO, "starting daemon (%d.%d):", MAJOR_VERSION, MINOR_VERSION);
-    syslog(LOG_INFO, "int=%ds realtime=%s sync=%s soft=%s mla=%d mem=%ld",
+    syslog(LOG_INFO, "int=%ds realtime=%s sync=%s soft=%s mla=%d mem=%d",
 	    tint,
 	    realtime ? "yes" : "no",
 	    sync_it ? "yes" : "no",
@@ -647,7 +662,7 @@ int main(int argc, char *const argv[])
             for (act = iface; act != NULL; act = act->next)
                 syslog(LOG_INFO, "interface: %s", act->name);                
 
-    syslog(LOG_INFO, "test=%s(%d) repair=%s alive=%s heartbeat=%s temp=%s to=%s no_act=%s",
+    syslog(LOG_INFO, "test=%s(%ld) repair=%s alive=%s heartbeat=%s temp=%s to=%s no_act=%s",
 	    (tbinary == NULL) ? "none" : tbinary, timeout, 
 	    (rbinary == NULL) ? "none" : rbinary,
 	    (devname == NULL) ? "none" : devname,
@@ -670,6 +685,17 @@ int main(int argc, char *const argv[])
 	    /* do not exit here per default */
 	    /* we can use watchdog even if there is no watchdog device */
 	}
+	if (watchdog >= 0 && devtimeout > 0) {
+	    /* Set the watchdog hard-stop timeout; default = unset (use
+	       driver default) */
+	    if (ioctl(watchdog, WDIOC_SETTIMEOUT, &devtimeout) < 0) {
+#if USE_SYSLOG
+            	syslog(LOG_ERR, "cannot set timeout %s (errno = %d = '%m')", strerror(errno), devtimeout, errno);
+#else				
+            	perror(progname);
+#endif			   
+	    }
+	}
     }
 
     /* MJ 16/2/2000, need to keep track of the watchdog writes so that
@@ -689,7 +715,7 @@ int main(int argc, char *const argv[])
             /* Allocate  memory for keeping the timestamps in */
             nrts = 0;
             lastts = 0;
-            timestamps = (unsigned char *) calloc(hbstamps, TS_SIZE);
+            timestamps = (char *) calloc(hbstamps, TS_SIZE);
             if ( timestamps == NULL ) {
 #if USE_SYSLOG
                 syslog(LOG_ERR, "cannot allocate memory for timestamps (errno = %d = '%m')", errno);
@@ -704,7 +730,8 @@ int main(int argc, char *const argv[])
                     memcpy(timestamps + (TS_SIZE * lastts), rbuf, TS_SIZE);
                     if (nrts < hbstamps) 
                         nrts++;
-                    lastts = ++lastts % hbstamps;
+                    ++lastts;
+                    lastts = lastts % hbstamps;
                 }
                 /* Write an indication that the watchdog has started to the heartbeat file */
                 /* copy it to the buffer */
@@ -714,7 +741,8 @@ int main(int argc, char *const argv[])
                 // success
                 if (nrts < hbstamps) 
                     nrts++;
-                lastts = ++lastts % hbstamps;
+                ++lastts;
+                lastts = lastts % hbstamps;
 
             }
         }
@@ -757,14 +785,15 @@ int main(int argc, char *const argv[])
     }
 
     /* tuck my process id away */
+    pid = getpid();
     fp = fopen(PIDFILE, "w");
     if (fp != NULL) {
-	fprintf(fp, "%d\n", pid = getpid());
+	fprintf(fp, "%d\n", pid);
 	(void) fclose(fp);
     }
-    /* set signal term to call terminate() */
-    /* to make sure watchdog device is closed */
-    signal(SIGTERM, terminate);
+    /* set signal term to set our run flag to 0 so that */
+    /* we make sure watchdog device is closed when receiving SIGTERM */
+    signal(SIGTERM, sigterm_handler);
 
 #if defined(_POSIX_MEMLOCK)
     if (realtime == TRUE) {
@@ -793,42 +822,42 @@ int main(int argc, char *const argv[])
 #endif
 
     /* main loop: update after <tint> seconds */
-    while (1) {
-	wd_action(keep_alive(), rbinary);
+    while (_running) {
+	wd_action(keep_alive(), rbinary, NULL);
 
 	/* sync system if we have to */
-	do_check(sync_system(sync_it), rbinary);
+	do_check(sync_system(sync_it), rbinary, NULL);
 
 	/* check file table */
-	do_check(check_file_table(), rbinary);
+	do_check(check_file_table(), rbinary, NULL);
 
 	/* check load average */
-	do_check(check_load(), rbinary);
+	do_check(check_load(), rbinary, NULL);
 	
 	/* check free memory */
-	do_check(check_memory(), rbinary);
+	do_check(check_memory(), rbinary, NULL);
 
 	/* check temperature */
-	do_check(check_temp(), rbinary);
+	do_check(check_temp(), rbinary, NULL);
 
 	/* in filemode stat file */
 	for (act = file; act != NULL; act = act->next)
-	    do_check(check_file_stat(act), rbinary);
+	    do_check(check_file_stat(act), rbinary, act->name);
 	    
 	/* in pidmode kill -0 processes */
 	for (act = pidfile; act != NULL; act = act->next)
-	    do_check(check_pidfile(act), rbinary);
+	    do_check(check_pidfile(act), rbinary, act->name);
 
 	/* in network mode check the given devices for input */
 	for (act = iface; act != NULL; act = act->next)
-	    do_check(check_iface(act), rbinary);
+	    do_check(check_iface(act), rbinary, act->name);
 	    
 	/* in ping mode ping the ip address */
 	for (act = target; act != NULL; act = act->next)
-	    do_check(check_net(act->name, act->parameter.net.sock_fp, act->parameter.net.to, act->parameter.net.packet, tint , pingcount), rbinary);
+	    do_check(check_net(act->name, act->parameter.net.sock_fp, act->parameter.net.to, act->parameter.net.packet, tint , pingcount), rbinary, act->name);
 
 	/* in user mode execute the given binary or just test fork() call */
-	do_check(check_bin(tbinary, timeout), rbinary);
+	do_check(check_bin(tbinary, timeout), rbinary, NULL);
 
 	/* finally sleep some seconds */
 	sleep((tint >> 1) + (tint % 2)); /* this should make watchdog sleep tint seconds alltogther */
@@ -843,4 +872,7 @@ int main(int argc, char *const argv[])
 	}
 #endif				/* USE_SYSLOG */
     }
+
+    terminate();
+    /* not reached */
 }
